import unittest
from kataCheckout.kataCheckout import Checkout, Product, Rules

class MyTestCase(unittest.TestCase):
    def setUp(self):
        # setting products
        self.A = Product('A', 50)
        self.B = Product('B', 30)
        self.C = Product('C', 20)
        self.D = Product('D', 15)
        self.F = Product('F', 0)
        # setting rules
        self.rules = {
             'A': Rules(self.A, quantity=3, discount=130),
             'B': Rules(self.B, quantity=2, discount=45),
             'C': Rules(self.C),
             'D': Rules(self.D)
        }
        # Initalize the kataCheckout
        self.checkout = Checkout(self.rules)

    def test_scan_single_item(self):
        self.checkout.scan(self.A)
        self.assertEqual(self.checkout.items[self.A], 1)

    def test_scan_multiple_items(self):
        self.checkout.scan(self.A)
        self.checkout.scan(self.A)
        self.checkout.scan(self.A)
        self.assertEqual(self.checkout.items[self.A], 3)

    def test_total_without_discount(self):
        self.checkout.scan(self.D)
        self.assertEqual(self.checkout.total(), 15)

    def test_total_with_discoutn(self):
        self.checkout.scan(self.A)
        self.checkout.scan(self.A)
        self.checkout.scan(self.A)
        self.assertEqual(self.checkout.total(), 130)

    def test_total_multipled_items(self):
        self.checkout.scan(self.A)
        self.checkout.scan(self.B)
        self.checkout.scan(self.B)
        self.checkout.scan(self.C)
        self.checkout.scan(self.D)
        self.checkout.scan(self.A)
        self.checkout.scan(self.A)
        self.assertEqual(self.checkout.total(), 210)

    def test_total_with_no_passed_items(self):
        self.assertEqual(self.checkout.total(), 0)

    def test_item_not_in_rules_lists(self):
        self.assertEqual(self.checkout.scan(self.F), "Item not in list")

if __name__ == '__main__':
    unittest.main()



# Domain Objects
class Product:
    def __init__(self, name, price):
        self.__name = name
        self.__price = price

    @property
    def name(self):
        return self.__name

    @property
    def price(self):
        return self.__price

    @price.setter
    def price(self, value):
        if value < 0:
            raise ValueError("Price cannot be negative")
        self.__price = value

class Rules:
    def __init__(self, product, quantity=None, discount=None):
        self.product = product
        self.quantity = quantity
        self.discount = discount

# Checkout
class Checkout:
    def __init__(self, rules):
        self.rules = rules
        self.items = {}

    def scan(self, product):
        if product.name not in self.rules:
            return "Item not in list"
        if product in self.items:
            self.items[product] += 1
        else:
            self.items[product] = 1

    def total(self):
        total= 0
        for product, count in self.items.items():
            rule = self.rules.get(product.name)
            if rule and rule.quantity:
                quantity, discount = rule.quantity, rule.discount
                total += discount * (count //quantity)
                count %= quantity
            total += count * product.price
        return total


from setuptools import setup, find_packages

VERSION = '2.0.2'
DESCRIPTION = 'Python package for kataCheckout'
LONG_DESCRIPTION = 'This is a python package for the kata 09: back to the checkout, which is a kata from codewars.com'

# Setting up
setup(
    # the name must match the folder name 'verysimplemodule'
    name="kataCheckout",
    version=VERSION,
    author="Alejandro Beltre",
    author_email="alejandro.beltre134@gmail.com",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[],

    keywords=['python', 'first package'],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)
